---
title: 初始化容器类
---

本节分析容器类，

```php
$container = require BASE_PATH . '/config/container.php';
```
这是容器类初始化的代码，
```php
use Hyperf\Context\ApplicationContext;
use Hyperf\Di\Container;
use Hyperf\Di\Definition\DefinitionSourceFactory;

$container = new Container((new DefinitionSourceFactory())());

return ApplicationContext::setContainer($container);

```
> 首先，可以看出，先是`new`了 一个`DefinitionSourceFactory`类，然后使用方法的调用方式调用类，会触发该类的`__invoke`方法，最后返回的内容作为`Container`类的构造函数的参数，然后返回容器类。

### DefinitionSourceFactory类
先看该类做了那些工作，
```php
declare(strict_types=1);
namespace Hyperf\Di\Definition;

use Hyperf\Config\ProviderConfig;
use Hyperf\Di\Exception\Exception;

class DefinitionSourceFactory
{
    public function __invoke(): DefinitionSource
    {
        // 常量不存在，报错
        if (! defined('BASE_PATH')) {
            throw new Exception('BASE_PATH is not defined.');
        }

        // 该方法上面一节分析过，获取所有的服务提供者返回的配置信息数组
        $configFromProviders = [];
        if (class_exists(ProviderConfig::class)) {
            $configFromProviders = ProviderConfig::load();
        }

        // 同理，这里会获取配置文件中的依赖配置信息，如果存在相同的key，则覆盖掉
        $serverDependencies = $configFromProviders['dependencies'] ?? [];
        $dependenciesPath = BASE_PATH . '/config/autoload/dependencies.php';
        if (file_exists($dependenciesPath)) {
            $definitions = include $dependenciesPath;
            $serverDependencies = array_replace($serverDependencies, $definitions ?? []);
        }
        // 返回DefinitionSource对象
        return new DefinitionSource($serverDependencies);
    }
}

```
只有一个方法，也是会被调用的`__invoke`方法。这里返回实例化的`DefinitionSource`对象。
### DefinitionSource类
> `DefinitionSource类`是管理容器中的绑定关系，即将抽象接口或类绑定到其对应的实现类或实例，来看源码。
```php
namespace Hyperf\Di\Definition;

use Hyperf\Di\ReflectionManager;
use ReflectionFunctionAbstract;
use ReflectionNamedType;

use function class_exists;
use function interface_exists;
use function is_callable;
use function is_string;
use function method_exists;

class DefinitionSource implements DefinitionSourceInterface
{
    protected array $source;

    public function __construct(array $source)
    {
        // source 是服务提供者配置文件中的依赖配置信息
        $this->source = $this->normalizeSource($source);
    }

    /**
     * Normalize the user definition source to a standard definition source.
     */
    protected function normalizeSource(array $source): array
    {
        $definitions = [];
        foreach ($source as $identifier => $definition) {
            $normalizedDefinition = $this->normalizeDefinition($identifier, $definition);
            if (! is_null($normalizedDefinition)) {
                $definitions[$identifier] = $normalizedDefinition;
            }
        }
        return $definitions;
    }

    /**
     * 绑定的内容可以是数组、类名、闭包
     * @param array|callable|string $definition
     */
    protected function normalizeDefinition(string $identifier, $definition): ?DefinitionInterface
    {
        if ($definition instanceof PriorityDefinition) {
            $definition = $definition->getDefinition();
        }

        if (is_string($definition) && class_exists($definition)) {
            if (method_exists($definition, '__invoke')) {
                return new FactoryDefinition($identifier, $definition, []);
            }
            return $this->autowire($identifier, new ObjectDefinition($identifier, $definition));
        }

        if (is_callable($definition)) {
            return new FactoryDefinition($identifier, $definition, []);
        }

        return null;
    }
    /**
     * 根据类名自动完成构造函数的依赖注入
     */
    protected function autowire(string $name, ObjectDefinition $definition = null): ?ObjectDefinition
    {
        $className = $definition ? $definition->getClassName() : $name;
        if (! class_exists($className) && ! interface_exists($className)) {
            return $definition;
        }

        $definition = $definition ?: new ObjectDefinition($name);

        /**
         * Constructor.
         */
        $class = ReflectionManager::reflectClass($className);
        $constructor = $class->getConstructor();
        if ($constructor && $constructor->isPublic()) {
            $constructorInjection = new MethodInjection('__construct', $this->getParametersDefinition($constructor));
            $definition->completeConstructorInjection($constructorInjection);
        }

        return $definition;
    }
}

```
> `normalizeSource`方法会将绑定的内容进行标准化，标准化后的内容是`DefinitionInterface`接口的实现类。（意思就是将绑定的`key`和`value`转换成`DefinitionInterface`接口的实现类）

最终，将绑定和对应的实现类保存到`$this->source`属性中。

实际上，这里`(new DefinitionSourceFactory())()`执行过后返回的是一个`DefinitionSource`对象，该对象中保存了所有的绑定关系。

然后继续往下看，
```php
$container = new Container((new DefinitionSourceFactory())());
```
> 这里返回`DefinitionSource`对象后，再调用`new Container`方法，传入`DefinitionSource`对象，返回`Container`对象。

查看`Container`类的构造函数源码，
```php
public function __construct(protected Definition\DefinitionSourceInterface $definitionSource)
{
    $this->definitionResolver = new ResolverDispatcher($this);
    // Auto-register the container.
    $this->resolvedEntries = [
        self::class => $this,
        PsrContainerInterface::class => $this,
        HyperfContainerInterface::class => $this,
    ];
}
```
总结：
1. 将传入的`$definitionSource`对象放到`$this->definitionSource`属性中。
2. 创建`ResolverDispatcher`解析器，调度不同类型的解析器来处理容器中的定义，它会根据定义的类型（如类、工厂函数等）来选择适当的解析器进行解析
3. 注册容器本身的实例

### 总结
1. 首先创建一个`DefinitionSourceFactory`工厂实例，并且触发该工厂实例的`__invoke`方法
2. `__invoke`方法中，首先加载配置文件包括类映射和依赖配置
3. 跟`config`目录下`dependencies.php`文件中的依赖配置进行合并
4. 创建`DefinitionSource`对象，传入依赖配置信息,负责管理容器中的绑定关系和依赖注入的定义
5. 实例化容器对象，将`DefinitionSource`对象传入
6. 构造函数中实例化`ResolverDispatcher`调度器作为容器类的属性，并注册容器本身的实例