---
title: 响应
---

```php
// $this->dispatcher 是实例化时传入的HttpDispatcher对象
$psr7Response = $this->dispatcher->dispatch($psr7Request, $middlewares, $this->coreMiddleware);
```
`$this->coreMiddleware`传入的是一个`CoreMiddleware`对象,
`$middlewares`传入的是配置的中间件数组。

> 文件位置: /vendor/hyperf/dispatcher/src/HttpDispatcher.php
```php
public function dispatch(...$params): ResponseInterface
{
    /**
     * @param RequestInterface $request
     * @param array $middlewares
     * @param MiddlewareInterface $coreHandler
     */
    [$request, $middlewares, $coreHandler] = $params;
    // 实例化
    $requestHandler = new HttpRequestHandler($middlewares, $coreHandler, $this->container);
    return $requestHandler->handle($request);
}
```
`dispatch`方法实例化`HttpRequestHandler`对象，并调用`handle`方法。

### HttpRequestHandler类
```php
$requestHandler = new HttpRequestHandler($middlewares, $coreHandler, $this->container);
```
该类没有构造方法，在父类中定义了`__construct`方法，接收三个参数。
```php
public function __construct(protected array $middlewares, protected $coreHandler, protected ContainerInterface $container)
{   // 将中间件数组的value取出
    $this->middlewares = array_values($this->middlewares);
}
```
实例化后调用`handle`方法。
```php
public function handle(ServerRequestInterface $request): ResponseInterface
{
    return $this->handleRequest($request);
}
```
`handleRequest`方法，这里执行所有的中间件，全部执行完成之后，会执行`CoreMiddleware`对象中的`process`方法。
```php
protected function handleRequest($request)
{
    // 如果中间件不存在，则执行CoreMiddleware对象中的process方法
    if (! isset($this->middlewares[$this->offset])) {
        $handler = $this->coreHandler;
    } else {
        // 中间件存在，取出并实例化中间件
        $handler = $this->middlewares[$this->offset];
        is_string($handler) && $handler = $this->container->get($handler);
    }
    // 如果中间件不存在或者没有process方法，则抛出异常
    if (! $handler || ! method_exists($handler, 'process')) {
        throw new InvalidArgumentException('Invalid middleware, it has to provide a process() method.');
    }
    // 调用中间件的process方法
    return $handler->process($request, $this->next());
}

protected function next(): self
{
    ++$this->offset;
    return $this;
}
```

#### CoreMiddleware的process方法
> 文件位置： /vendor/hyperf/http-server/src/CoreMiddleware.php
```php
public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
{
    // 将请求对象设置到上下文中
    $request = RequestContext::set($request);

    /** @var Dispatched $dispatched */
    $dispatched = $request->getAttribute(Dispatched::class);

    if (! $dispatched instanceof Dispatched) {
        throw new ServerException(sprintf('The dispatched object is not a %s object.', Dispatched::class));
    }

    // 检查是否匹配到路由
    $response = match ($dispatched->status) {
        Dispatcher::NOT_FOUND => $this->handleNotFound($request), // 会抛出NotFoundHttpException异常
        Dispatcher::METHOD_NOT_ALLOWED => $this->handleMethodNotAllowed($dispatched->params, $request), // 会抛出MethodNotAllowedHttpException异常
        Dispatcher::FOUND => $this->handleFound($dispatched, $request),
        default => null,
    };

    if (! $response instanceof ResponsePlusInterface) {
        $response = $this->transferToResponse($response, $request);
    }
    // 添加服务器信息
    return $response->addHeader('Server', 'Hyperf');
}
```

当匹配到路由时，执行`handleFound`方法。
`dispatched`对象结构如下，
![]( https://oss.xiaokeaii.top/2024/dispatched.png)
```php
protected function handleFound(Dispatched $dispatched, ServerRequestInterface $request): mixed
{   
    // 闭包情况
    if ($dispatched->handler->callback instanceof Closure) {
        $parameters = $this->parseClosureParameters($dispatched->handler->callback, $dispatched->params);
        $callback = $dispatched->handler->callback;
        $response = $callback(...$parameters);
    } else {
        // 控制器情况
        [$controller, $action] = $this->prepareHandler($dispatched->handler->callback);
        $controllerInstance = $this->container->get($controller);
        // 控制器方法不存在抛异常
        if (! method_exists($controllerInstance, $action)) {
            throw new ServerErrorHttpException('Method of class does not exist.');
        }
        // 解析控制器方法参数
        $parameters = $this->parseMethodParameters($controller, $action, $dispatched->params);
        // 调用控制器方法
        $response = $controllerInstance->{$action}(...$parameters);
    }
    // 返回响应
    return $response;
}
```
##### 控制器类型分析
`prepareHandler`方法解析出控制器类和控制器方法。
```php
protected function prepareHandler(string|array $handler): array
{
    if (is_string($handler)) {
        if (str_contains($handler, '@')) {
            return explode('@', $handler);
        }
        if (str_contains($handler, '::')) {
            return explode('::', $handler);
        }
        return [$handler, '__invoke'];
    }
    if (is_array($handler) && isset($handler[0], $handler[1])) {
        return $handler;
    }
    throw new RuntimeException('Handler not exist.');
}
```
解析出控制器实例。
```php
$controllerInstance = $this->container->get($controller);
```

##### 闭包类型分析
解析闭包参数，调用闭包，返回响应。
```php
$parameters = $this->parseClosureParameters($dispatched->handler->callback, $dispatched->params);
$callback = $dispatched->handler->callback;
$response = $callback(...$parameters);
```

`handleFound`方法走完，获取到响应对象。
##### 转换响应对象
如果响应对象不是`ResponsePlusInterface`接口的实现类，需要进行转换。
```php
if (! $response instanceof ResponsePlusInterface) {
    $response = $this->transferToResponse($response, $request);
}

protected function transferToResponse($response, ServerRequestInterface $request): ResponsePlusInterface
{
    if (is_string($response)) {
        return $this->response()->addHeader('content-type', 'text/plain')->setBody(new SwooleStream($response));
    }

    if ($response instanceof ResponseInterface) {
        return new ResponsePlusProxy($response);
    }

    if (is_array($response) || $response instanceof Arrayable) {
        return $this->response()
            ->addHeader('content-type', 'application/json')
            ->setBody(new SwooleStream(Json::encode($response)));
    }

    if ($response instanceof Jsonable) {
        return $this->response()
            ->addHeader('content-type', 'application/json')
            ->setBody(new SwooleStream((string) $response));
    }

    if ($this->response()->hasHeader('content-type')) {
        return $this->response()->setBody(new SwooleStream((string) $response));
    }

    return $this->response()->addHeader('content-type', 'text/plain')->setBody(new SwooleStream((string) $response));
}
```

1. 如果是字符串，则转换为文本类型的响应对象，并设置相应的头部信息和响应体
2. 如果是`ResponseInterface`的实现类，则返回`ResponsePlusProxy`对象
3. 如果是数组或者`Arrayable`的实现类，则转换为`JSON`类型的响应对象，并设置相应的头部信息和响应体
4. 如果是实现了`Jsonable`的接口，则转换为`JSON`类型的响应对象，并设置相应的头部信息和响应体
5. 如果响应已经设置了`content-type`头部信息，则将其转换为对应的响应对象
6. 如果以上条件都不满足，则将其视为文本类型的响应对象

返回响应对象。

### 发送响应到客户端

```php
if (isset($psr7Request) && $psr7Request->getMethod() === 'HEAD') {
    $this->responseEmitter->emit($psr7Response, $response, false);
} else {
    $this->responseEmitter->emit($psr7Response, $response);
}
```
#### emit方法
发送响应到客户端。
```php
public function emit(ResponseInterface $response, mixed $connection, bool $withContent = true): void
{
  try {
      // 检查连接对象的 Upgrade 头部信息是否为 websocket，如果是，则直接返回，不发送响应内容
      if (strtolower($connection->header['Upgrade'] ?? '') === 'websocket') {
          return;
      }
      // 将响应对象转换为 Swoole 响应对象
      $this->buildSwooleResponse($connection, $response);
      // 获取响应对象的主体内容
      $content = $response->getBody();
      // 如果主体内容是文件，则使用 sendfile 方法发送文件内容
      if ($content instanceof FileInterface) {
          $connection->sendfile($content->getFilename());
          return;
      }
      // 是否发送响应内容
      if ($withContent) {
          $connection->end((string) $content);
      } else {
          $connection->end();
      }
  } catch (Throwable $exception) {
      $this->logger?->critical((string) $exception);
  }
}
```